@@ -6,56 +6,42 @@
 
 #include "ais.h"
 
+Ais8::Ais8(const char *nmea_payload, const size_t pad)
+    : AisMsg(nmea_payload, pad) {
+  // Ais8 is used in some apps as a standalone, so be extra
+  // careful to make sure we have the lookup table built
+  assert(nmea_ord_initialized);
+  if (status != AIS_UNINITIALIZED)
+    return;
 
-Ais8::Ais8(const char *nmea_payload, const size_t pad) {
-    assert(nmea_payload);
-    assert(pad < 6);
-    // Ais8 is used in some apps as a standalone, so be extra
-    // careful to make sure we have the lookup table built
-    assert(nmea_ord_initialized);
-    init();
+  assert(message_id == 8);
 
-    // in bits w/o DAC/FI
-    const int payload_len = strlen(nmea_payload)*6 - 46 - pad;
-    if (payload_len < 0 || payload_len > 952) {
-        status = AIS_ERR_BAD_BIT_COUNT;
-        return;
-    }
+  // in bits w/o DAC/FI
+  const int payload_len = strlen(nmea_payload) * 6 - 46 - pad;
+  if (payload_len < 0 || payload_len > 952) {
+    status = AIS_ERR_BAD_BIT_COUNT;
+    return;
+  }
 
-    bitset<MAX_BITS> bs;
-    status = aivdm_to_bits(bs, nmea_payload);
-    if (had_error()) return;
+  bitset<MAX_BITS> bs;
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
-    if (!decode_header8(bs)) return;
-
-    // Handle all the byte aligned payload
-    for (int i = 0; i < payload_len/8; i++) {
-        const int start = 56 + i*8;
-        payload.push_back(ubits(bs, start, 8));
-    }
-
-    // TODO(schwehr): need to handle spare bits.
-    const int remainder = payload_len % 8;
-    if (remainder > 0) {
-        const int start = (payload_len/8) * 8;
-        payload.push_back(ubits(bs, start, remainder));
-    }
+  spare = ubits(bs, 38, 2);
+  dac = ubits(bs, 40, 10);
+  fi = ubits(bs, 50, 6);
 }
 
-bool Ais8::decode_header8(const bitset<MAX_BITS> &bs) {
-    message_id = ubits(bs, 0, 6);
-    if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return false; }
-    repeat_indicator = ubits(bs, 6, 2);
-    mmsi = ubits(bs, 8, 30);
-    spare = ubits(bs, 38, 2);
-    dac = ubits(bs, 40, 10);
-    fi = ubits(bs, 50, 6);
-    return true;
-}
+Ais8_1_0::Ais8_1_0(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
 
-Ais8_1_0::Ais8_1_0(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+  assert(dac == 1);
+  assert(fi == 0);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -65,24 +51,18 @@ Ais8_1_0::Ais8_1_0(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<MAX_BITS> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) { return; }  // checks status
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 0 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
   ack_required = bs[56];
   msg_seq = ubits(bs, 57, 11);
 
   const size_t text_size = 6 * ((num_bits - 68)/6);
-  const size_t spare2_size = num_bits - 68 - text_size;  // wrong?  needs to land on 8-bit boundary
+  // wrong?  needs to land on 8-bit boundary
+  const size_t spare2_size = num_bits - 68 - text_size;
   text = ais_str(bs, 68, text_size);
 
   // TODO(schwehr): Is this correct?
@@ -90,107 +70,101 @@ Ais8_1_0::Ais8_1_0(const char *nmea_payload, const size_t pad) {
     spare2 = 0;
   else
     spare2 = ubits(bs, 68, spare2_size);
+  status = AIS_OK;
 }
 
+Ais8_1_11::Ais8_1_11(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
 
-Ais8_1_11::Ais8_1_11(const char *nmea_payload, const size_t pad) {
-    assert(nmea_payload);  assert(pad <= 7);
-    init();
+  assert(dac == 1);
+  assert(fi == 11);
 
-    if (strlen(nmea_payload) != 59) {
-      status = AIS_ERR_BAD_BIT_COUNT;
-      return;
-    }
+  if (strlen(nmea_payload) != 59) {
+    status = AIS_ERR_BAD_BIT_COUNT;
+    return;
+  }
 
-    bitset<354> bs;  // 352 + 2 spares to be 6 bit aligned
-    status = aivdm_to_bits(bs, nmea_payload);
-    if (had_error()) return;
+  bitset<354> bs;  // 352 + 2 spares to be 6 bit aligned
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
-    message_id = ubits(bs, 0, 6);
-    if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-    repeat_indicator = ubits(bs, 6, 2);
-    mmsi = ubits(bs, 8, 30);
+  y = sbits(bs, 56, 24) / 60000.;
+  x = sbits(bs, 80, 25) / 60000.;
+  day = ubits(bs, 105, 5);
+  hour = ubits(bs, 110, 5);
+  minute = ubits(bs, 115, 6);
+  wind_ave = ubits(bs, 121, 7);
+  wind_gust = ubits(bs, 128, 7);
+  wind_dir = ubits(bs, 135, 9);
+  wind_gust_dir = ubits(bs, 144, 9);
+  air_temp = ubits(bs, 153, 11) / 10. - 60;
+  rel_humid = ubits(bs, 164, 7);
+  dew_point = ubits(bs, 171, 10) / 10. - 20;  // TODO(schwehr): verify
+  air_pres = ubits(bs, 181, 9) + 800;
+  air_pres_trend = ubits(bs, 190, 2);
+  horz_vis = ubits(bs, 192, 8) / 10.;
+  // TODO(schwehr): verify for -10.0 to 30.0
+  water_level = ubits(bs, 200, 9) / 10. - 10;
+  water_level_trend = ubits(bs, 209, 2);
+  surf_cur_speed = ubits(bs, 211, 8) / 10.;
+  surf_cur_dir = ubits(bs, 219, 9);
+  cur_speed_2 = ubits(bs, 228, 8) / 10.;
+  cur_dir_2 = ubits(bs, 236, 9);
+  cur_depth_2 = ubits(bs, 245, 5);
+  cur_speed_3 = ubits(bs, 250, 8) / 10.;
+  cur_dir_3 = ubits(bs, 258, 9);
+  cur_depth_3 = ubits(bs, 267, 5);
 
-    spare = ubits(bs, 38, 2);
-    dac = ubits(bs, 40, 10);
-    fi = ubits(bs, 50, 6);
+  wave_height = ubits(bs, 272, 8) / 10.;
+  wave_period = ubits(bs, 280, 6);
+  wave_dir = ubits(bs, 286, 9);
+  swell_height = ubits(bs, 295, 8) / 10.;
+  swell_period = ubits(bs, 303, 6);
+  swell_dir = ubits(bs, 309, 9);
 
-    if (1 != dac || 11 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  sea_state = ubits(bs, 318, 4);
+  // TODO(schwehr): verify for -10.0 to +50.0
+  water_temp = ubits(bs, 322, 10) / 10. - 10;
+  precip_type = ubits(bs, 332, 3);
+  salinity = ubits(bs, 335, 9);
+  ice = ubits(bs, 344, 2);
+  // There is no way to know which meaning to attach to the following 6 bits
+  // TODO(schwehr): how to treat this spare vrs water level?
+  spare2 = ubits(bs, 346, 6);
+  extended_water_level = ubits(bs, 346, 6);
 
-    y = sbits(bs, 56, 24) / 60000.;
-    x = sbits(bs, 80, 25) / 60000.;
-    day = ubits(bs, 105, 5);
-    hour = ubits(bs, 110, 5);
-    minute = ubits(bs, 115, 6);
-    wind_ave = ubits(bs, 121, 7);
-    wind_gust = ubits(bs, 128, 7);
-    wind_dir = ubits(bs, 135, 9);
-    wind_gust_dir = ubits(bs, 144, 9);
-    air_temp = ubits(bs, 153, 11) / 10. - 60;
-    rel_humid = ubits(bs, 164, 7);
-    dew_point = ubits(bs, 171, 10) / 10. - 20;  // TODO(schwehr): verify
-    air_pres = ubits(bs, 181, 9) + 800;
-    air_pres_trend = ubits(bs, 190, 2);
-    horz_vis = ubits(bs, 192, 8) / 10.;
-    // TODO(schwehr): verify for -10.0 to 30.0
-    water_level = ubits(bs, 200, 9) / 10. - 10;
-    water_level_trend = ubits(bs, 209, 2);
-    surf_cur_speed = ubits(bs, 211, 8) / 10.;
-    surf_cur_dir = ubits(bs, 219, 9);
-    cur_speed_2 = ubits(bs, 228, 8) / 10.;
-    cur_dir_2 = ubits(bs, 236, 9);
-    cur_depth_2 = ubits(bs, 245, 5);
-    cur_speed_3 = ubits(bs, 250, 8) / 10.;
-    cur_dir_3 = ubits(bs, 258, 9);
-    cur_depth_3 = ubits(bs, 267, 5);
-
-    wave_height = ubits(bs, 272, 8) / 10.;
-    wave_period = ubits(bs, 280, 6);
-    wave_dir = ubits(bs, 286, 9);
-    swell_height = ubits(bs, 295, 8) / 10.;
-    swell_period = ubits(bs, 303, 6);
-    swell_dir = ubits(bs, 309, 9);
-
-    sea_state = ubits(bs, 318, 4);
-    // TODO(schwehr): verify for -10.0 to +50.0
-    water_temp = ubits(bs, 322, 10) / 10. - 10;
-    precip_type = ubits(bs, 332, 3);
-    salinity = ubits(bs, 335, 9);
-    ice = ubits(bs, 344, 2);
-    // There is no way to know which meaning to attach to the following 6 bits
-    // TODO(schwehr): how to treat this spare vrs water level?
-    spare2 = ubits(bs, 346, 6);
-    extended_water_level = ubits(bs, 346, 6);
+  status = AIS_OK;
 }
 
 // No 8_1_12
 
 // IMO Circ 289 - Fairway Closed
 // See also Circ 236
-Ais8_1_13::Ais8_1_13(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_1_13::Ais8_1_13(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+  assert(dac == 1);
+  assert(fi == 13);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
-  if (472 != num_bits) {
+  if (num_bits != 472) {
     status = AIS_ERR_BAD_BIT_COUNT;
     return;
   }
 
   bitset<472> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 13 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   reason = ais_str(bs, 56, 120);
   location_from = ais_str(bs, 176, 120);
@@ -206,50 +180,20 @@ Ais8_1_13::Ais8_1_13(const char *nmea_payload, const size_t pad) {
   hour_to = ubits(bs, 457, 5);
   minute_to = ubits(bs, 462, 6);
   spare2 = ubits(bs, 468, 4);
+  status = AIS_OK;
 }
 
-
 // No 8_1_16
 
-
 // IMO Circ 289 - Extended Shipdata - Air gap
 // See also Circ 236
-Ais8_1_15::Ais8_1_15(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  assert(pad < 6);
-  init();
-
-  const size_t num_bits = strlen(nmea_payload) * 6 - pad;
-
-  if (72 != num_bits) {
-    status = AIS_ERR_BAD_BIT_COUNT;
+Ais8_1_15::Ais8_1_15(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
     return;
-  }
 
-  bitset<72> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 15 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-
-  air_draught = ubits(bs, 56, 11) / 10.;
-  spare2 = ubits(bs, 67, 5);
-}
-
-// IMO Circ 289 - Number of persons on board
-// See also Circ 236
-// TODO(schwehr): there might also be an addressed version?
-Ais8_1_16::Ais8_1_16(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+  assert(dac == 1);
+  assert(fi == 15);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -259,28 +203,56 @@ Ais8_1_16::Ais8_1_16(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<72> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
+  air_draught = ubits(bs, 56, 11) / 10.;
+  spare2 = ubits(bs, 67, 5);
+  status = AIS_OK;
+}
 
-  if (1 != dac || 16 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+// IMO Circ 289 - Number of persons on board
+// See also Circ 236
+// TODO(schwehr): there might also be an addressed version?
+Ais8_1_16::Ais8_1_16(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 16);
+
+  const size_t num_bits = strlen(nmea_payload) * 6 - pad;
+
+  if (num_bits != 72) {
+    status = AIS_ERR_BAD_BIT_COUNT;
+    return;
+  }
+
+  bitset<72> bs;
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   persons = ubits(bs, 56, 13);
   spare2 = ubits(bs, 69, 3);
+  status = AIS_OK;
 }
 
 // IMO Circ 289 - VTS Generated/Synthetic Targets
 // See also Circ 236
-Ais8_1_17::Ais8_1_17(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_1_17::Ais8_1_17(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 17);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
   const size_t num_targets = (num_bits - 56) / 120;
@@ -292,18 +264,11 @@ Ais8_1_17::Ais8_1_17(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<536> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 17 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   for (size_t target_num = 0; target_num < num_targets; target_num++) {
     Ais8_1_17_Target target;
@@ -317,16 +282,19 @@ Ais8_1_17::Ais8_1_17(const char *nmea_payload, const size_t pad) {
     target.timestamp = ubits(bs, start + 106, 6);
     target.sog = ubits(bs, start + 112, 8);
   }
+  status = AIS_OK;
 }
 
-
 // No msg 8_1_18
 
 // IMO Circ 289 - Marine traffic signal
-Ais8_1_19::Ais8_1_19(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  assert(pad < 6);
-  init();
+Ais8_1_19::Ais8_1_19(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 19);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -337,18 +305,11 @@ Ais8_1_19::Ais8_1_19(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<360> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 19 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   link_id = ubits(bs, 56, 10);
   name = ais_str(bs, 66, 120);
@@ -359,21 +320,26 @@ Ais8_1_19::Ais8_1_19(const char *nmea_payload, const size_t pad) {
   utc_hour_next = ubits(bs, 242, 5);
   utc_min_next = ubits(bs, 247, 6);
   next_signal = ubits(bs, 253, 5);
-  if (360 != num_bits) return;
+  if (num_bits != 360)
+    return;
   spare2[0] = ubits(bs, 258, 32);
   spare2[1] = ubits(bs, 290, 32);
   spare2[2] = ubits(bs, 322, 32);
   spare2[3] = ubits(bs, 354, 6);
+  status = AIS_OK;
 }
 
 // No 8_1_20
 
 // IMO Circ 289 - Weather observation report from ship
 // See also Circ 236
-Ais8_1_21::Ais8_1_21(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  assert(pad < 6);
-  init();
+Ais8_1_21::Ais8_1_21(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 21);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -383,19 +349,11 @@ Ais8_1_21::Ais8_1_21(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<360> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  // TODO(schwehr): what counties use their own dac/fi waters?  Please do NOT do that.
-  if (1 != dac || 21 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   type_wx_report = bs[56];
   if (!type_wx_report) {
@@ -450,8 +408,8 @@ Ais8_1_21::Ais8_1_21(const char *nmea_payload, const size_t pad) {
     // TODO(schwehr): change this to celcius
     air_temp_raw = ubits(bs, 195, 10);
     humidity =ubits(bs, 205, 7);
-    water_temp_raw = ubits(bs, 212, 9);  // TODO(schwehr): change this to C.  Jerks.
-    horz_viz = pow(ubits(bs, 221, 6), 2) * 13.073;  // WTF?  the result is meters.
+    water_temp_raw = ubits(bs, 212, 9);  // TODO(schwehr): change this to C
+    horz_viz = pow(ubits(bs, 221, 6), 2) * 13.073;  // m
     wx[0] = ubits(bs, 227, 9);  // current
     wx[1] = ubits(bs, 236, 5);  // past 1
     wx[2] = ubits(bs, 241, 5);  // past 2
@@ -460,7 +418,7 @@ Ais8_1_21::Ais8_1_21(const char *nmea_payload, const size_t pad) {
     cloud_low_type = ubits(bs, 254, 6);
     cloud_middle_type = ubits(bs, 260, 6);
     cloud_high_type = ubits(bs, 266, 6);
-    alt_lowest_cloud_base = pow(ubits(bs, 272, 7), 2) * 0.16;  // Jerks
+    alt_lowest_cloud_base = pow(ubits(bs, 272, 7), 2) * 0.16;
     wave_period = ubits(bs, 279, 5);  // s
     wave_height = ubits(bs, 284, 6) * 0.5;  // m
     swell_dir = ubits(bs, 290, 6) * 10;
@@ -478,14 +436,19 @@ Ais8_1_21::Ais8_1_21(const char *nmea_payload, const size_t pad) {
     ice_devel = ubits(bs, 351, 5);
     bearing_ice_edge = ubits(bs, 356, 4) * 45;
   }
+  status = AIS_OK;
 }
 
 
 // IMO Circ 289 - Extended ship static and voyage-related
 // See also Circ 236
-Ais8_1_24::Ais8_1_24(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_1_24::Ais8_1_24(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 24);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -495,18 +458,11 @@ Ais8_1_24::Ais8_1_24(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<360> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 24 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   link_id = ubits(bs, 56, 10);
   air_draught = ubits(bs, 66, 13) / 10.;  // m
@@ -531,6 +487,7 @@ Ais8_1_24::Ais8_1_24(const char *nmea_payload, const size_t pad) {
   bunker_oil = ubits(bs, 323, 14);  // tonnes
   persons = ubits(bs, 337, 13);
   spare2 = ubits(bs, 350, 10);
+  status = AIS_OK;
 }
 
 
@@ -542,10 +499,13 @@ Ais8_1_24::Ais8_1_24(const char *nmea_payload, const size_t pad) {
 
 // IMO Circ 289 - Route information
 // See also Circ 236
-Ais8_1_27::Ais8_1_27(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  assert(pad < 6);
-  init();
+Ais8_1_27::Ais8_1_27(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 27);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
   const size_t num_waypoints = (num_bits - 117) / 55;
@@ -557,18 +517,11 @@ Ais8_1_27::Ais8_1_27(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<997> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 27 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   link_id = ubits(bs, 56, 10);
   sender_type = ubits(bs, 66, 3);
@@ -587,17 +540,21 @@ Ais8_1_27::Ais8_1_27(const char *nmea_payload, const size_t pad) {
     pt.y =  sbits(bs, start + 28, 27) / 600000.;
     waypoints.push_back(pt);
   }
-}
 
+  status = AIS_OK;
+}
 
 // No 8_1_28
 
-
 // IMO Circ 289 - Text description
 // See also Circ 236
-Ais8_1_29::Ais8_1_29(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_1_29::Ais8_1_29(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 29);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -607,18 +564,11 @@ Ais8_1_29::Ais8_1_29(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<1032> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) {status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 29 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   link_id = ubits(bs, 56, 10);
   size_t text_bits = num_bits - 66;
@@ -630,15 +580,21 @@ Ais8_1_29::Ais8_1_29(const char *nmea_payload, const size_t pad) {
   } else {
     spare2 = 0;
   }
+
+  status = AIS_OK;
 }
 
 
 
 // IMO Circ 289 - Meteorological and Hydrographic data
 // See also Circ 236
-Ais8_1_31::Ais8_1_31(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_1_31::Ais8_1_31(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 1);
+  assert(fi == 31);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -648,18 +604,11 @@ Ais8_1_31::Ais8_1_31(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<360> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 31 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   x = sbits(bs, 56, 25) / 60000.;
   y = sbits(bs, 81, 24) / 60000.;
@@ -704,14 +653,20 @@ Ais8_1_31::Ais8_1_31(const char *nmea_payload, const size_t pad) {
   salinity = ubits(bs, 339, 9) / 10.;
   ice = ubits(bs, 348, 2);  // yes/no/undef/unknown
   spare2 = ubits(bs, 350, 10);
+
+  status = AIS_OK;
 }
 
 
 // River Information Systems ECE-TRANS-SC3-2006-10r-RIS.pdf
 // Inland ship static and voyage related data
-Ais8_200_10::Ais8_200_10(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_200_10::Ais8_200_10(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 200);
+  assert(fi == 10);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
   if (num_bits != 168) {
@@ -720,18 +675,11 @@ Ais8_200_10::Ais8_200_10(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<168> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 10 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   eu_id = ais_str(bs, 56, 48);
   length = ubits(bs, 104, 13) / 10.;  // m
@@ -744,13 +692,18 @@ Ais8_200_10::Ais8_200_10(const char *nmea_payload, const size_t pad) {
   course_qual = bs[158];
   heading_qual = bs[159];
   spare2 = ubits(bs, 160, 8);
+
+  status = AIS_OK;
 }
 
 // River Information Systems ECE-TRANS-SC3-2006-10r-RIS.pdf
-Ais8_200_23::Ais8_200_23(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  assert(pad < 6);
-  init();
+Ais8_200_23::Ais8_200_23(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 200);
+  assert(fi == 23);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -760,18 +713,11 @@ Ais8_200_23::Ais8_200_23(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<256> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 23 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   utc_year_start = ubits(bs, 56, 9);
   utc_month_start = ubits(bs, 65, 4);
@@ -796,34 +742,33 @@ Ais8_200_23::Ais8_200_23(const char *nmea_payload, const size_t pad) {
   classification = ubits(bs, 244, 2);
   wind_dir = ubits(bs, 246, 4);
   spare2 = ubits(bs, 250, 6);
+
+  status = AIS_OK;
 }
 
 
 // River Information Systems ECE-TRANS-SC3-2006-10r-RIS.pdf
-Ais8_200_24::Ais8_200_24(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_200_24::Ais8_200_24(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 200);
+  assert(fi == 24);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
-  if (168 != num_bits) {
+  if (num_bits != 168) {
     status = AIS_ERR_BAD_BIT_COUNT;
     return;
   }
 
   bitset<168> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 24 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   ais_str(bs, 56, 12);
   for (size_t i = 0; i < 4; i++) {
@@ -833,33 +778,31 @@ Ais8_200_24::Ais8_200_24(const char *nmea_payload, const size_t pad) {
     // ERROR: the spec has a bit listing mistake
     levels[i] = sign * ubits(bs, start + 12, 13);
   }
+  status = AIS_OK;
 }
 
 // River Information Systems ECE-TRANS-SC3-2006-10r-RIS.pdf
-Ais8_200_40::Ais8_200_40(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_200_40::Ais8_200_40(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 200);
+  assert(fi == 40);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
-  if (168 != num_bits) {
+  if (num_bits != 168) {
     status = AIS_ERR_BAD_BIT_COUNT;
     return;
   }
 
   bitset<168> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 40 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   x = sbits(bs, 56, 28) / 600000.;
   y = sbits(bs, 84, 27) / 600000.;
@@ -869,13 +812,19 @@ Ais8_200_40::Ais8_200_40(const char *nmea_payload, const size_t pad) {
   status_raw = ubits(bs, 127, 30);
   // TODO(schwehr): status[ ] = bite me;
   spare2 = ubits(bs, 157, 11);
+
+  status = AIS_OK;
 }
 
 
 // River Information Systems ECE-TRANS-SC3-2006-10r-RIS.pdf
-Ais8_200_55::Ais8_200_55(const char *nmea_payload, const size_t pad) {
-  assert(nmea_payload);
-  init();
+Ais8_200_55::Ais8_200_55(const char *nmea_payload, const size_t pad)
+    : Ais8(nmea_payload, pad) {
+  if (status != AIS_UNINITIALIZED)
+    return;
+
+  assert(dac == 200);
+  assert(fi == 55);
 
   const size_t num_bits = strlen(nmea_payload) * 6 - pad;
 
@@ -887,18 +836,11 @@ Ais8_200_55::Ais8_200_55(const char *nmea_payload, const size_t pad) {
   }
 
   bitset<168> bs;
-  status = aivdm_to_bits(bs, nmea_payload);
-  if (had_error()) return;
-
-  message_id = ubits(bs, 0, 6);
-  if (8 != message_id) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
-  repeat_indicator = ubits(bs, 6, 2);
-  mmsi = ubits(bs, 8, 30);
-  spare = ubits(bs, 38, 2);
-  dac = ubits(bs, 40, 10);
-  fi = ubits(bs, 50, 6);
-
-  if (1 != dac || 55 != fi) { status = AIS_ERR_WRONG_MSG_TYPE; return; }
+  const AIS_STATUS r = aivdm_to_bits(bs, nmea_payload);
+  if (r != AIS_OK) {
+    status = r;
+    return;
+  }
 
   crew = ubits(bs, 56, 8);
   passengers = ubits(bs, 64, 13);
@@ -917,4 +859,6 @@ Ais8_200_55::Ais8_200_55(const char *nmea_payload, const size_t pad) {
     spare2[1] = ubits(bs, 117, 19);
     spare2[2] = 0;
   }
+
+  status = AIS_OK;
 }
